% this handles the "Freq Response" action for DALE

 % enable clear command
 set(h_clr, 'Enable','on');

 % input test_equalizer if exists 
 if ishandle(h_trace_f), eqs_exist = 1; else, eqs_exist = 0; end;
 if ishandle(h_trace_c),
   set(h_msg,'BackgroundColor',[0 1 0]);
   set(h_msg,'FontAngle','italic', 'String','click on surface...'); drawnow;
   figure(fig_surf);
   zoom off; f_freq = ginput(1).'; 			% get eqlzer directly
   hold on; h_trace_new = plot(f_freq(1),f_freq(2),'o'); hold off; zoom on;
   h_trace_f = [h_trace_f,h_trace_new];
   set(h_trace_new,'Color',color); drawnow;
   set(h_msg,'BackgroundColor',[.8 .8 .8]);
   set(h_msg, 'FontAngle','normal', 'String',''); drawnow;
   test_equalizer = 1;
 elseif (ishandle(h_trace_h))&(N_f == length(f(:,1))),	
 % careful with invalid histories: equalizer may be wrong length!
   set(h_msg,'BackgroundColor',[0 1 0]);
   set(h_msg,'FontAngle','italic', 'String','click on history...'); drawnow;
   figure(fig_hist);
   set(h_trace_mse,'HandleVisibility','off');	% avoid legend 
   zoom off; [N_freq,dum] = ginput(1); zoom on;		% get iteration number
   set(h_trace_mse,'HandleVisibility','on');
   f_freq = f(:,floor(N_freq));				% use equalizer history 
   hold on; 
   h_trace_new = plot(D_e*round(N_freq/D_e),e_filt(round(N_freq/D_e)),'o'); 
   hold off;
   h_trace_f = [h_trace_f,h_trace_new];
   set(h_trace_new,'Color',color); drawnow;
   set(h_msg,'BackgroundColor',[.8 .8 .8]);
   set(h_msg, 'FontAngle','normal', 'String',''); drawnow;
   test_equalizer = 1;
 else,
   test_equalizer = 0;
 end; 


 % calculate known frequency responses
 N_fft = 512;
 C_freq = 20*log10(abs(fft(c,N_fft))+eps);
 F_des = 20*log10(abs(fft(f_des,N_fft))+eps);
 H_des = 20*log10(abs(fft(C*f_des,N_fft))+eps);

 % avoid matlab idiosynchrocies
 if max(abs(H_des))<1e-10, H_des = zeros(size(H_des)); end;	
 if max(abs(F_des))<1e-10, F_des = zeros(size(F_des)); end;	
 if max(abs(C_freq))<1e-10, C_freq = zeros(size(C_freq)); end;	
 if (length(H_des)==1)&(N_fft~=1), H_des = H_des*ones(1,N_fft); end;

 % replot known frequency responses only if necessary
 if (isempty(h_trace_ff))|(min(ishandle(h_trace_ff))==0),
   figure(fig_freq); 
   set(fig_freq, 'DeleteFcn','ui_del_freq');
   subplot(211);		% channel response (possible FS)
     if eqs_exist, hold on; end;
     h_trace_ff = plot([0:N_fft/2-1]/N_fft*(1+is_FSE),C_freq(1:N_fft/2),'r-.');
     hold off;
   subplot(212);		% optimal system (definitely BS)
     % this is really a dummy trace, but it makes things easier...
     if eqs_exist, hold on; end;
     h_trace_ff = [h_trace_ff,...
	plot([0:N_fft/2-1]/N_fft,H_des(1:N_fft/2),'k--')];
     hold off;
 end

 if test_equalizer,		% calculate and plot frequency responses
   F_freq = 20*log10(abs(fft(f_freq,N_fft))+eps);
   H_freq = 20*log10(abs(fft(C*f_freq,N_fft))+eps);
   % fix a matlab idiosynchracy...
   if (length(H_freq)==1)&(N_fft~=1), H_freq = H_freq*ones(1,N_fft); end;
   figure(fig_freq);
   subplot(211);		% test equalizer (possible FS)
     hold on;
     h_trace_new = plot([0:N_fft/2-1]/N_fft*(1+is_FSE),F_freq(1:N_fft/2));
     h_trace_f = [h_trace_f,h_trace_new];
     set(h_trace_new,'Color',color);
     hold off;
     %legend refresh done later...
   subplot(212);		% test system (definitely BS)
     hold on;
     h_trace_new = plot([0:N_fft/2-1]/N_fft,H_freq(1:N_fft/2));
     h_trace_f = [h_trace_f,h_trace_new];
     set(h_trace_new,'Color',color);
     hold off;
 end

 % overlay dashed traces (redoing the BS system trace) so they are visible 
 figure(fig_freq);
 subplot(211);
   delete(h_trace_ff(2));		% dont want to accumulate these
   hold on;
   h_trace_ff(2)=plot([0:N_fft/2-1]/N_fft*(1+is_FSE),F_des(1:N_fft/2),'k--');
   hold off;
   ylabel('dB'); xlabel('normalized to baud frequency');
   title('Channel, equalizer frequency response magnitudes');
   legend(h_trace_ff([1,2]),'channel','eq_{opt}'); 
   zoom on;
 subplot(212);
   if length(h_trace_ff)==3, 		% dont want to accumulate these
     delete(h_trace_ff(3)); h_trace_ff = h_trace_ff(1:2); 
   end;
   hold on; 
   h_trace_ff = [h_trace_ff,...
	plot([0:N_fft/2-1]/N_fft,H_des(1:N_fft/2),'k--')];
   hold off;
   ylabel('dB'); xlabel('normalized to baud frequency');
   title('System frequency response magnitude');
   legend(h_trace_ff(3),'system_{opt}');
   zoom on;
